@@ -60,7 +60,8 @@ struct _Type {
 	char *subtype;
 
 	/* Contains xmlNodes for elements that are being copied to the output.
-	 * That is, <comment> nodes and anything with an unknown namespace.
+	 * That is, <comment>, <sub-class-of> and <alias> nodes, and anything
+	 * with an unknown namespace.
 	 */
 	xmlDoc	*output;
 };
@@ -288,7 +289,22 @@ static gboolean process_freedesktop_node(Type *type, xmlNode *field,
 			g_return_val_if_fail(magic == NULL, FALSE);
 	}
 	else if (strcmp(field->name, "comment") == 0)
-		return FALSE;	/* Copy through */
+		return FALSE;
+	else if (strcmp(field->name, "alias") == 0 ||
+		 strcmp(field->name, "sub-class-of") == 0)
+	{
+		char *other_type;
+		gboolean valid;
+		other_type = xmlGetNsProp(field, "type", NULL);
+		valid = other_type && strchr(other_type, '/');
+		xmlFree(other_type);
+		if (valid)
+			return FALSE;	/* Copy through */
+
+		g_set_error(error, MIME_ERROR, 0,
+			_("Incorrect or missing 'type' attribute "
+			  "in <%s>"), field->name);
+	}
 	else if (strcmp(field->name, "root-XML") == 0)
 	{
 		char *namespaceURI, *localName;
@@ -786,7 +802,7 @@ static void getstr(const char *s, GString *out)
  */
 static void parse_int_value(int bytes, const char *in, const char *in_mask,
 			    GString *parsed_value, char **parsed_mask,
-			    GError **error)
+			    gboolean big_endian, GError **error)
 {
 	char *end;
 	char *out_mask = NULL;
@@ -802,7 +818,7 @@ static void parse_int_value(int bytes, const char *in, const char *in_mask,
 
 	for (b = 0; b < bytes; b++)
 	{
-		int shift = (bytes - b - 1) * 8;
+		int shift = (big_endian ? (bytes - b - 1) : b) * 8;
 		g_string_append_c(parsed_value, (value >> shift) & 0xff);
 	}
 
@@ -910,13 +926,13 @@ static void parse_value(const char *type, const char *in, const char *in_mask,
 
 	if (strstr(type, "16"))
 		parse_int_value(2, in, in_mask, parsed_value, parsed_mask,
-				error);
+				type[0] != 'l', error);
 	else if (strstr(type, "32"))
 		parse_int_value(4, in, in_mask, parsed_value, parsed_mask,
-				error);
+				type[0] != 'l', error);
 	else if (strcmp(type, "byte") == 0)
 		parse_int_value(1, in, in_mask, parsed_value, parsed_mask,
-				error);
+				FALSE, error);
 	else if (strcmp(type, "string") == 0)
 	{
 		getstr(in, parsed_value);
@@ -1357,7 +1373,7 @@ out:
 
 int main(int argc, char **argv)
 {
-	const char *mime_dir = NULL;
+	char *mime_dir = NULL;
 	char *package_dir = NULL;
 	int opt;
 
@@ -1390,6 +1406,17 @@ int main(int argc, char **argv)
 	LIBXML_TEST_VERSION;
 
 	mime_dir = argv[optind];
+
+	/* Strip trailing / characters */
+	{
+		int l = strlen(mime_dir);
+		while (l > 1 && mime_dir[l - 1] == '/')
+		{
+			l--;
+			mime_dir[l] = '\0';
+		}
+	}
+
 	package_dir = g_strconcat(mime_dir, "/packages", NULL);
 
 	if (access(mime_dir, W_OK))
